@@ -1,7 +1,7 @@
 package Guacamole;
 our $AUTHORITY = 'cpan:XSAWYERX';
 # ABSTRACT: A parser toolkit for Standard Perl
-$Guacamole::VERSION = '0.004';
+$Guacamole::VERSION = '0.005';
 use strict;
 use warnings;
 use Marpa::R2;
@@ -177,12 +177,9 @@ ExprShiftU    ::= ExprShiftU  OpShift   ExprAddU        | ExprAddU     action =>
 ExprShift0    ::= ExprShiftU  OpShift   ExprAdd0        | ExprAdd0     action => ::first
 ExprShiftL    ::= ExprShiftU  OpShift   ExprAddL        | ExprAddL     action => ::first
 ExprShiftR    ::= ExprShiftU  OpShift   ExprAddR        | ExprAddR     action => ::first
-ExprFile0     ::= OpFile      ExprFile0                 | ExprShift0 action => ::first
-ExprFileL     ::= OpFile      ExprFileL                 | ExprShiftL action => ::first
-ExprFileR     ::= OpFile      ExprFileR                 | ExprShiftR action => ::first
-ExprNeq0      ::= ExprFile0   OpInequal ExprFile0       | ExprFile0    action => ::first
-ExprNeqL      ::= ExprFile0   OpInequal ExprFileL       | ExprFileL    action => ::first
-ExprNeqR      ::= ExprFile0   OpInequal ExprFileR       | ExprFileR    action => ::first
+ExprNeq0      ::= ExprShift0  OpInequal ExprShift0      | ExprShift0   action => ::first
+ExprNeqL      ::= ExprShift0  OpInequal ExprShiftL      | ExprShiftL   action => ::first
+ExprNeqR      ::= ExprShift0  OpInequal ExprShiftR      | ExprShiftR   action => ::first
 ExprEq0       ::= ExprNeq0    OpEqual   ExprNeq0        | ExprNeq0     action => ::first
 ExprEqL       ::= ExprNeq0    OpEqual   ExprNeqL        | ExprNeqL     action => ::first
 ExprEqR       ::= ExprNeq0    OpEqual   ExprNeqR        | ExprNeqR     action => ::first
@@ -253,12 +250,9 @@ NonBraceExprShiftU    ::= NonBraceExprShiftU  OpShift   ExprAddU        | NonBra
 NonBraceExprShift0    ::= NonBraceExprShiftU  OpShift   ExprAdd0        | NonBraceExprAdd0     action => ::first
 NonBraceExprShiftL    ::= NonBraceExprShiftU  OpShift   ExprAddL        | NonBraceExprAddL     action => ::first
 NonBraceExprShiftR    ::= NonBraceExprShiftU  OpShift   ExprAddR        | NonBraceExprAddR     action => ::first
-NonBraceExprFile0     ::= OpFile      ExprFile0                 | NonBraceExprShift0 action => ::first
-NonBraceExprFileL     ::= OpFile      ExprFileL                 | NonBraceExprShiftL action => ::first
-NonBraceExprFileR     ::= OpFile      ExprFileR                 | NonBraceExprShiftR action => ::first
-NonBraceExprNeq0      ::= NonBraceExprFile0   OpInequal ExprFile0       | NonBraceExprFile0    action => ::first
-NonBraceExprNeqL      ::= NonBraceExprFile0   OpInequal ExprFileL       | NonBraceExprFileL    action => ::first
-NonBraceExprNeqR      ::= NonBraceExprFile0   OpInequal ExprFileR       | NonBraceExprFileR    action => ::first
+NonBraceExprNeq0      ::= NonBraceExprShift0  OpInequal ExprShift0      | NonBraceExprShift0   action => ::first
+NonBraceExprNeqL      ::= NonBraceExprShift0  OpInequal ExprShiftL      | NonBraceExprShiftL   action => ::first
+NonBraceExprNeqR      ::= NonBraceExprShift0  OpInequal ExprShiftR      | NonBraceExprShiftR   action => ::first
 NonBraceExprEq0       ::= NonBraceExprNeq0    OpEqual   ExprNeq0        | NonBraceExprNeq0     action => ::first
 NonBraceExprEqL       ::= NonBraceExprNeq0    OpEqual   ExprNeqL        | NonBraceExprNeqL     action => ::first
 NonBraceExprEqR       ::= NonBraceExprNeq0    OpEqual   ExprNeqR        | NonBraceExprNeqR     action => ::first
@@ -543,6 +537,7 @@ ArrowDerefCall     ::= CallArgs
 ArrowDerefVariable ::= DerefVariableArgsAll
                      | DerefVariableSlice
 ArrowMethodCall    ::= SubNameExpr CallArgs
+                     | SubNameExpr
 ArrowIndirectCall  ::= SigilScalar VarIdentExpr CallArgs
 
 DerefVariableArgsAll ::= '$*' | '@*' | '%*' | '&*' | '**' | '$#*'
@@ -679,6 +674,7 @@ OpUnaryKeywordExpr ::=
     | OpKeywordUntieExpr
     | OpKeywordUtimeExpr
     | OpKeywordValuesExpr
+    | OpFileExpr
 
 OpListKeywordExpr ::=
       OpKeywordAcceptExpr
@@ -1290,6 +1286,10 @@ OpFile ::=
     | OpFileAccessTime
     | OpFileChangeTime
 
+OpFileExpr ::= OpFile OpFileArg
+OpFileArg  ::= OpUnaryKeywordArg
+             | BuiltinFilehandle
+
 QLikeValue ::= QLikeValueExpr | QLikeValueExprWithMods
 
 QLikeValueExpr
@@ -1299,6 +1299,7 @@ QLikeValueExpr
     | QLikeFunction '[' NonRBracketOrEscapedBrackets_Any           ']'
     | QLikeFunction '/' NonForwardSlashOrEscapedForwardSlashes_Any '/'
     | QLikeFunction '!' NonExclamPointOrEscapedExclamPoints_Any    '!'
+    | QLikeFunction '|' NonPipeOrEscapedPipes_Any                  '|'
 
 QLikeFunction ~ OpKeywordQ
               | OpKeywordQq
@@ -1315,12 +1316,14 @@ QLikeValueExprWithMods
     | QLikeFunctionWithMods '[' NonRBracketOrEscapedBrackets_Any           ']' RegexModifiers
     | QLikeFunctionWithMods '/' NonForwardSlashOrEscapedForwardSlashes_Any '/' RegexModifiers
     | QLikeFunctionWithMods '!' NonExclamPointOrEscapedExclamPoints_Any    '!' RegexModifiers
+    | QLikeFunctionWithMods '|' NonPipeOrEscapedPipes_Any                  '|' RegexModifiers
     | QLikeSubstWithMods    '(' NonRParenOrEscapedParens_Any               ')(' NonRParenOrEscapedParens_Any               ')' RegexModifiers
     | QLikeSubstWithMods    '{' NonRBraceOrEscapedBraces_Any               '}{' NonRBraceOrEscapedBraces_Any               '}' RegexModifiers
     | QLikeSubstWithMods    '<' NonRAngleOrEscapedAngles_Any               '><' NonRAngleOrEscapedAngles_Any               '>' RegexModifiers
-    | QLikeSubstWithMods    '[' NonRBracketOrEscapedBrackets_Any           '][' NonRBracketOrEscapedBrackets_Any             ']' RegexModifiers
+    | QLikeSubstWithMods    '[' NonRBracketOrEscapedBrackets_Any           '][' NonRBracketOrEscapedBrackets_Any           ']' RegexModifiers
     | QLikeSubstWithMods    '/' NonForwardSlashOrEscapedForwardSlashes_Any '/'  NonForwardSlashOrEscapedForwardSlashes_Any '/' RegexModifiers
     | QLikeSubstWithMods    '!' NonExclamPointOrEscapedExclamPoints_Any    '!'  NonExclamPointOrEscapedExclamPoints_Any    '!' RegexModifiers
+    | QLikeSubstWithMods    '|' NonPipeOrEscapedPipes_Any                  '|'  NonPipeOrEscapedPipes_Any                  '|' RegexModifiers
     | '/' NonForwardSlashOrEscapedForwardSlashes_Any '/' RegexModifiers
     | '`' NonBacktickOrEscapedBackticks_Any '`'
 
@@ -1451,6 +1454,11 @@ NonExclamPointOrEscapedExclamPoints     ~ EscapedExclamPoint | NonExclamPoint
 EscapedExclamPoint                      ~ Escape [!]
 NonExclamPoint                          ~ [^\!]
 
+NonPipeOrEscapedPipes_Any ~ NonPipeOrEscapedPipes*
+NonPipeOrEscapedPipes     ~ EscapedPipe | NonPipe
+EscapedPipe               ~ Escape [\|]
+NonPipe                   ~ [^\|]
+
 NonBacktickOrEscapedBackticks_Any ~ NonBacktickOrEscapedBackticks*
 NonBacktickOrEscapedBackticks     ~ EscapedBacktick | NonBacktick
 EscapedBacktick                   ~ Escape [`]
@@ -2085,20 +2093,140 @@ Guacamole - A parser toolkit for Standard Perl
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
     use Guacamole;
     my ($ast) = Guacamole->parse($string);
 
-=head1 WHERE'S THE REST
+=head1 DESCRIPITON
 
-Gotta write it... Give us time!
+B<Guacamole> is a Perl parser toolkit.
+
+It can:
+
+=over 4
+
+=item * Parse Standard Perl
+
+This is explained in this document.
+
+For B<Standard Perl>, see the next clause.
+
+=item * Check a file is written in Standard Perl
+
+This is done by L<standard>, which is where Standard Perl is described.
+
+=item * Lint your code
+
+See L<Guacamole::Linter>.
+
+=item * Deparse your code
+
+See L<Guacamole::Deparse>.
+
+=item * Rewrite your code
+
+There is a proof-of-concept for this and we hope to provide this as a framework.
+
+=back
+
+=head1 Standard Perl
+
+Guacamole only works on Standard Perl. You can read about it here: L<standard>.
+
+=head1 Parser
+
+    my ($ast) = Guacamole->parse($string);
+
+To parse a string, call L<Gaucamole>'s C<parse> method. (This might turn to an
+object-oriented interface in the future.)
+
+It returns a list of results. If it ever returns more than one, this is a bug that
+means it couldn't ambiguously parse something. This will later be enforced in the
+interface. The current interface is not official.
+
+=head2 AST Nodes
+
+Guacamole returns an AST with two types of nodes.
+
+    my ($ast) = Guacamole->parse('$foo = 1');
+
+The above will generate a larger AST than you imagine (which might be pruned
+in the future). We'll focus on two types of nodes that will appear above.
+
+=head3 Rules
+
+Rules are the top level expressions. They include the definitions for rules.
+They include information on location in the file, length, line, and column.
+
+    $rule = {
+        'children'  => [...],
+        'column'    => 2,
+        'length'    => 3,
+        'line'      => 1,
+        'name'      => 'VarIdentExpr',
+        'start_pos' => 1,
+        'type'      => 'rule',
+    },
+
+This rule is a C<VarIdentExpr> which is an expression for a variable identity.
+
+In the code above, it refers to the C<foo> in C<$foo> - which is the identity
+itself.
+
+It has one child, described below under C<Lexemes>.
+
+=head3 Lexemes
+
+The child for the C<VarIdentExpr> rule should be the value of the identity.
+
+    $lexeme = {
+        'name'  => '',
+        'type'  => 'lexeme',
+        'value' => 'foo',
+    };
+
+The C<name> attribute for all lexemes is empty. This is to make it easy to
+write code that checks for the value of a rule without having to check whether
+it's a rule first.
+
+=head1 THANKS
+
+=over 4
+
+=item * Damian Conway
+
+For helping understand what is feasible, what isn't, and why, and for having
+infinite patience in explaining these.
+
+=item * Jeffrey Kegler
+
+For L<Marpa> and helping understand how to use Marpa better.
+
+=item * Gonzalo Diethelm
+
+For continuous feedback and support.
+
+=item * H. Merijn Brand (@Tux)
+
+For providing the initial production-level test of Guacamole to
+help shake many of the bugs in the BNF.
+
+=back
 
 =head1 SEE ALSO
 
-L<standard>
+=over 4
+
+=item * L<standard>
+
+=item * L<Gaucamole::Linter>
+
+=item * L<Guacamole::Deparse>
+
+=back
 
 =head1 AUTHORS
 
